@@ -65,6 +65,7 @@ int main(int argc, char *argv[])
 	struct mddev_ident_s ident;
 	char *configfile = NULL;
 	char *cp;
+	char *update = NULL;
 	int scan = 0;
 	char devmode = 0;
 	int runstop = 0;
@@ -81,6 +82,8 @@ int main(int argc, char *argv[])
 	char *mailaddr = NULL;
 	char *program = NULL;
 	int delay = 0;
+	int daemonise = 0;
+	int oneshot = 0;
 
 	int mdfd = -1;
 
@@ -99,14 +102,18 @@ int main(int argc, char *argv[])
 		switch(opt) {
 		case 'h':
 			help_text = Help;
-			switch (mode) {
-			case ASSEMBLE : help_text = Help_assemble; break;
-			case BUILD    : help_text = Help_build; break;
-			case CREATE   : help_text = Help_create; break;
-			case MANAGE   : help_text = Help_manage; break;
-			case MISC     : help_text = Help_misc; break;
-			case MONITOR  : help_text = Help_monitor; break;
-			}
+			if (option_index > 0 && 
+			    strcmp(long_options[option_index].name, "help-options")==0)
+				help_text = OptionHelp;
+			else
+				switch (mode) {
+				case ASSEMBLE : help_text = Help_assemble; break;
+				case BUILD    : help_text = Help_build; break;
+				case CREATE   : help_text = Help_create; break;
+				case MANAGE   : help_text = Help_manage; break;
+				case MISC     : help_text = Help_misc; break;
+				case MONITOR  : help_text = Help_monitor; break;
+				}
 			fputs(help_text,stderr);
 			exit(0);
 
@@ -314,6 +321,12 @@ int main(int argc, char *argv[])
 					optarg);
 				exit(2);
 			}
+			if (raiddisks == 1 &&  !force) {
+				fprintf(stderr, Name ": '1' is an unusual number of drives for an array, so it is probably\n"
+					"     a mistake.  If you really mean it you will need to specify --force before\n"
+					"     setting the number of drives.\n");
+				exit(2);
+			}
 			ident.raid_disks = raiddisks;
 			continue;
 
@@ -335,6 +348,7 @@ int main(int argc, char *argv[])
 				exit(2);
 			}
 			continue;
+		case O(BUILD,'f'): /* force honouring '-n 1' */
 		case O(CREATE,'f'): /* force honouring of device list */
 		case O(ASSEMBLE,'f'): /* force assembly */
 		case O(MISC,'f'): /* force zero */
@@ -357,18 +371,35 @@ int main(int argc, char *argv[])
 			continue;
 
 		case O(ASSEMBLE,'m'): /* super-minor for array */
-			if (ident.super_minor >= 0) {
+			if (ident.super_minor != -1) {
 				fprintf(stderr, Name ": super-minor cannot be set twice.  "
 					"Second value: %s.\n", optarg);
 				exit(2);
 			}
-			ident.super_minor = strtoul(optarg, &cp, 10);
-			if (!optarg[0] || *cp) {
-				fprintf(stderr, Name ": Bad super-minor number: %s.\n", optarg);
-				exit(2);
+			if (strcmp(optarg, "dev")==0)
+				ident.super_minor = -2;
+			else {
+				ident.super_minor = strtoul(optarg, &cp, 10);
+				if (!optarg[0] || *cp) {
+					fprintf(stderr, Name ": Bad super-minor number: %s.\n", optarg);
+					exit(2);
+				}
 			}
 			continue;
 
+		case O(ASSEMBLE,'U'): /* update the superblock */
+			if (update) {
+				fprintf(stderr, Name ": Can only update one aspect of superblock, both %s and %s given.\n",
+					update, optarg);
+				exit(2);
+			}
+			update = optarg;
+			if (strcmp(update, "sparc2.2")==0) continue;
+			if (strcmp(update, "super-minor") == 0)
+				continue;
+			fprintf(stderr, Name ": '--update %s' invalid.  Only 'sparc2.2' or 'super-minor' supported\n",update);
+			exit(2);
+
 		case O(ASSEMBLE,'c'): /* config file */
 		case O(MISC, 'c'):
 		case O(MONITOR,'c'):
@@ -415,7 +446,12 @@ int main(int argc, char *argv[])
 				}
 			}
 			continue;
-			
+		case O(MONITOR,'f'): /* daemonise */
+			daemonise = 1;
+			continue;
+		case O(MONITOR,'1'): /* oneshot */
+			oneshot = 1;
+			continue;
 
 			/* now the general management options.  Some are applicable
 			 * to other modes. None have arguments.
@@ -489,13 +525,6 @@ int main(int argc, char *argv[])
 			}
 			SparcAdjust = 1;
 			continue;
-		case O(MISC,23):
-			if (devmode != 'E') {
-				fprintf(stderr, Name ": --sparc2.2update only allowed with --examine\n");
-				exit(2);
-			}
-			SparcAdjust = 2;
-			continue;
 		}
 		/* We have now processed all the valid options. Anything else is
 		 * an error
@@ -528,6 +557,11 @@ int main(int argc, char *argv[])
 		mdfd = open_mddev(devlist->devname);
 		if (mdfd < 0)
 			exit(1);
+		if (ident.super_minor == -2) {
+			struct stat stb;
+			fstat(mdfd, &stb);
+			ident.super_minor = MINOR(stb.st_rdev);
+		}
 	}
 
 	rv = 0;
@@ -545,11 +579,33 @@ int main(int argc, char *argv[])
 			rv = Manage_runstop(devlist->devname, mdfd, runstop);
 		break;
 	case ASSEMBLE:
-		if (!scan)
+		if (devs_found == 1 && ident.uuid_set == 0 &&
+		    ident.super_minor == -1 && !scan ) {
+			/* Only a device has been given, so get details from config file */
+			mddev_ident_t array_ident = conf_get_ident(configfile, devlist->devname);
+			mdfd = open_mddev(devlist->devname);
+			if (mdfd < 0)
+				rv |= 1;
+			else {
+				if (array_ident == NULL) {
+					fprintf(stderr, Name ": %s not identified in config file.\n",
+						devlist->devname);
+					rv |= 1;
+				}
+				else 
+					rv |= Assemble(devlist->devname, mdfd, array_ident, configfile,
+						       NULL,
+						       readonly, runstop, update, verbose, force);
+			}
+		} else if (!scan)
 			rv = Assemble(devlist->devname, mdfd, &ident, configfile,
 				      devlist->next,
-				      readonly, runstop, verbose, force);
-		else if (devs_found>0)
+				      readonly, runstop, update, verbose, force);
+		else if (devs_found>0) {
+			if (update && devs_found > 1) {
+				fprintf(stderr, Name ": can only update a single array at a time\n");
+				exit(1);
+			}
 			for (dv = devlist ; dv ; dv=dv->next) {
 				mddev_ident_t array_ident = conf_get_ident(configfile, dv->devname);
 				mdfd = open_mddev(dv->devname);
@@ -565,9 +621,9 @@ int main(int argc, char *argv[])
 				}
 				rv |= Assemble(dv->devname, mdfd, array_ident, configfile,
 					       NULL,
-					       readonly, runstop, verbose, force);
+					       readonly, runstop, update, verbose, force);
 			}
-		else {
+		} else {
 			mddev_ident_t array_list =  conf_get_ident(configfile, NULL);
 			if (!array_list) {
 				fprintf(stderr, Name ": No arrays found in config file\n");
@@ -586,7 +642,7 @@ int main(int argc, char *argv[])
 					rv |= Assemble(array_list->devname, mdfd,
 						       array_list, configfile,
 						       NULL,
-						       readonly, runstop, verbose, force);
+						       readonly, runstop, NULL, verbose, force);
 				}
 		}
 		break;
@@ -665,14 +721,13 @@ int main(int argc, char *argv[])
 		}
 		break;
 	case MONITOR:
-/*
 		if (!devlist && !scan) {
 			fprintf(stderr, Name ": Cannot monitor: need --scan or at least one device\n");
 			rv = 1;
 			break;
 		}
-*/		rv= Monitor(devlist, mailaddr, program,
-			    delay?delay:60, scan, configfile);
+		rv= Monitor(devlist, mailaddr, program,
+			    delay?delay:60, daemonise, scan, oneshot, configfile);
 		break;
 	}
 	exit(rv);
